#include "iMS_EnhancedSequences.h"
#include "Supervisors.h"

static bool wait_for_tone_buffer_download(IMSSystem& ims, const ToneBuffer& tb, int n_tones = 256)
{
	ToneBufferDownload* dl = new ToneBufferDownload(ims, tb);
	ToneBufferDownloadSupervisor ds;

	dl->ToneBufferDownloadEventSubscribe(ToneBufferEvents::DOWNLOAD_FINISHED, &ds);
	dl->ToneBufferDownloadEventSubscribe(ToneBufferEvents::DOWNLOAD_ERROR, &ds);

	if (!dl->StartDownload(tb.begin(), tb.begin() + n_tones)) {
		std::cout << "Error starting download" << std::endl;
		dl->ToneBufferDownloadEventUnsubscribe(ToneBufferEvents::DOWNLOAD_FINISHED, &ds);
		dl->ToneBufferDownloadEventUnsubscribe(ToneBufferEvents::DOWNLOAD_ERROR, &ds);
		delete dl;
		return false;
	}

	while (ds.Busy()) {
		std::this_thread::sleep_for(std::chrono::milliseconds(50));
	}

	dl->ToneBufferDownloadEventUnsubscribe(ToneBufferEvents::DOWNLOAD_FINISHED, &ds);
	dl->ToneBufferDownloadEventUnsubscribe(ToneBufferEvents::DOWNLOAD_ERROR, &ds);

	delete dl;
	if (ds.Error())
	{
		std::cout << "Tone Buffer Download FAILED. " << std::endl;
		return false;
	}

	return true;
}

static ToneBuffer tb;

void createAndDownloadToneMap(std::shared_ptr<iMS::IMSSystem> myiMS)
{
	MHz uf = myiMS->Synth().GetCap().upperFrequency;
	MHz lf = myiMS->Synth().GetCap().lowerFrequency;

	for (unsigned int i = 0; i < n_tones; i++)
	{
		// Ascending equally spaced tones starting at 50MHz, covering half frequency range of Synthesiser
		tb[i].SetAll(FAP(MHz(50.0 + (((double)i / (double)(n_tones - 1)) * (uf - lf) / 2.0)), Percent(100.0), Degrees(0.0)));
		
		// Set Tone Buffer Sync Digital output lower bit alternating 1 / 0 - demonstrates how tone buffer can be used to turn on/off external equipment
		tb[i].SetSyncD(i & 1);
	}
	std::cout << "Downloading Tone Buffer..." << std::flush;
	if (!wait_for_tone_buffer_download(*myiMS, tb, n_tones)) {
		std::cout << "Aborting.. <press any key>" << std::endl;
		char c;
		std::cin >> c;
		std::exit(-1);
	}
}

bool matchesToneBuffer(std::array<uint8_t, 16> uuid) {
	return (tb.UUID() == uuid);
}

const ToneBuffer& getToneMap() {
	return tb;
}